const dbSocketPath = process.env.DB_SOCKET_PATH || "/cloudsql";
const SECOND = 1000;

function reporter(message, options) {
  console.log(message);
}
const MINUTE = SECOND * 60;
const config = {
  development: {
    username: process.env.DB_USER,
    database: process.env.DB_NAME,
    host: dbSocketPath + "/" + process.env.MASTER_DB_INSTANCE_URL,
    dialect: "postgres",
    port: 5432,
    seederStorage: "sequelize",
    seederStorageTableName: "SequelizeData",
    dialectOptions: {
      socketPath: dbSocketPath + "/" + process.env.MASTER_DB_INSTANCE_URL,
    },
    benchmark: true,
    pool: {
      max: 5,
      idle: SECOND * 10,
      acquire: MINUTE * 2,
    },
    retry: {
      max: 6, // maximum amount of tries
      timeout: MINUTE * 8, // throw if no response or error within millisecond timeout
      match: [
        // Must match error signature (ala bluebird catch) to continue
        /ConnectionError/gim,
        /SequelizeConnectionError/gim,
        /SequelizeConnectionTimedOutError/gim,
        /SequelizeConnectionAcquireTimeoutError/gim,
        /connection error/gim,
        /recovery mode/gim,
      ],
      backoffBase: SECOND, // Initial backoff duration in ms. Default: 100,
      backoffExponent: 1.1, // Exponent to increase backoff each try. Default: 1.1
      report: reporter,
      name: "SequelizeRetry", // if user supplies string, it will be used when composing error/reporting messages; else if retry gets a callback, uses callback name in erroring/reporting; else (default) uses literal string 'unknown'
    }
  },
  beta: {
    username: process.env.DB_USER,
    database: process.env.DB_NAME,
    host: dbSocketPath + "/" + process.env.MASTER_DB_INSTANCE_URL,
    dialect: "postgres",
    port: 5432,
    seederStorage: "sequelize",
    seederStorageTableName: "SequelizeData",
    dialectOptions: {
      socketPath: dbSocketPath + "/" + process.env.MASTER_DB_INSTANCE_URL,
    },
    benchmark: true,
    pool: {
      max: 5,
      idle: SECOND * 10,
      acquire: MINUTE * 2,
    },
    retry: {
      max: 6, // maximum amount of tries
      timeout: MINUTE * 8, // throw if no response or error within millisecond timeout
      match: [
        // Must match error signature (ala bluebird catch) to continue
        /ConnectionError/gim,
        /SequelizeConnectionError/gim,
        /SequelizeConnectionTimedOutError/gim,
        /SequelizeConnectionAcquireTimeoutError/gim,
        /connection error/gim,
        /recovery mode/gim,
      ],
      backoffBase: SECOND, // Initial backoff duration in ms. Default: 100,
      backoffExponent: 1.1, // Exponent to increase backoff each try. Default: 1.1
      report: reporter,
      name: "SequelizeRetry", // if user supplies string, it will be used when composing error/reporting messages; else if retry gets a callback, uses callback name in erroring/reporting; else (default) uses literal string 'unknown'
    }
  },
  production: {
    username: process.env.DB_USER,
    database: process.env.DB_NAME,
    host: dbSocketPath + "/" + process.env.MASTER_DB_INSTANCE_URL,
    dialect: "postgres",
    port: 5432,
    seederStorage: "sequelize",
    seederStorageTableName: "SequelizeData",
    dialectOptions: {
      socketPath: dbSocketPath + "/" + process.env.MASTER_DB_INSTANCE_URL,
    },
    benchmark: true,
    pool: {
      max: 5,
      idle: SECOND * 10,
      acquire: MINUTE * 2,
    },
    retry: {
      max: 6, // maximum amount of tries
      timeout: MINUTE * 8, // throw if no response or error within millisecond timeout
      match: [
        // Must match error signature (ala bluebird catch) to continue
        /ConnectionError/gim,
        /SequelizeConnectionError/gim,
        /SequelizeConnectionTimedOutError/gim,
        /SequelizeConnectionAcquireTimeoutError/gim,
        /connection error/gim,
        /recovery mode/gim,
      ],
      backoffBase: SECOND, // Initial backoff duration in ms. Default: 100,
      backoffExponent: 1.1, // Exponent to increase backoff each try. Default: 1.1
      report: reporter,
      name: "SequelizeRetry", // if user supplies string, it will be used when composing error/reporting messages; else if retry gets a callback, uses callback name in erroring/reporting; else (default) uses literal string 'unknown'
    }
  },
  local: {
    username: process.env.DB_USER,
    database: process.env.DB_NAME,
    host: process.env.MASTER_DB_INSTANCE_URL,
    dialect: "postgres",
    port: 5432,
    seederStorage: "sequelize",
    seederStorageTableName: "SequelizeData",
    benchmark: true,
    pool: {
      max: 5,
      idle: SECOND * 10,
      acquire: MINUTE * 2,
    },
    retry: {
      max: 6, // maximum amount of tries
      timeout: MINUTE * 8, // throw if no response or error within millisecond timeout
      match: [
        // Must match error signature (ala bluebird catch) to continue
        /ConnectionError/gim,
        /SequelizeConnectionError/gim,
        /SequelizeConnectionTimedOutError/gim,
        /SequelizeConnectionAcquireTimeoutError/gim,
        /connection error/gim,
        /recovery mode/gim,
      ],
      backoffBase: SECOND, // Initial backoff duration in ms. Default: 100,
      backoffExponent: 1.1, // Exponent to increase backoff each try. Default: 1.1
      report: reporter,
      name: "SequelizeRetry", // if user supplies string, it will be used when composing error/reporting messages; else if retry gets a callback, uses callback name in erroring/reporting; else (default) uses literal string 'unknown'
    }
  },
  test: {
    username: process.env.DB_USER,
    database: process.env.DB_NAME,
    host: process.env.MASTER_DB_INSTANCE_URL,
    dialect: "postgres",
    port: 5432,
    seederStorage: "sequelize",
    seederStorageTableName: "SequelizeData",
    benchmark: true,
    pool: {
      max: 5,
      idle: SECOND * 10,
      acquire: MINUTE * 2,
    },
    retry: {
      max: 6, // maximum amount of tries
      timeout: MINUTE * 8, // throw if no response or error within millisecond timeout
      match: [
        // Must match error signature (ala bluebird catch) to continue
        /ConnectionError/gim,
        /SequelizeConnectionError/gim,
        /SequelizeConnectionTimedOutError/gim,
        /SequelizeConnectionAcquireTimeoutError/gim,
        /connection error/gim,
        /recovery mode/gim,
      ],
      backoffBase: SECOND, // Initial backoff duration in ms. Default: 100,
      backoffExponent: 1.1, // Exponent to increase backoff each try. Default: 1.1
      report: reporter,
      name: "SequelizeRetry", // if user supplies string, it will be used when composing error/reporting messages; else if retry gets a callback, uses callback name in erroring/reporting; else (default) uses literal string 'unknown'
    }
  },
};

// TODO: REmove Hard Coded Password
config.development.password = "HDMI6ha0wTBj11vEmMc24U";
config.beta.password = "HDMI6ha0wTBj11vEmMc24U";
config.production.password = "HDMI6ha0wTBj11vEmMc24U";
config.local.password = "HDMI6ha0wTBj11vEmMc24U";
config.test.password = "HDMI6ha0wTBj11vEmMc24U";

module.exports = config;

/*
----------------------------------------------
        SEQUELIZE RETRY CONFIGURATION
----------------------------------------------
backoffBase = 1000
backoffExponent = 1.1
MAX RETRY = 6

Try   #0 
Retry #1 delayed by 1 sec    | 1000 (backoffBase)
Retry #2 delayed by 1.9 sec  | 1000^1.1 
Retry #3 delayed by 4.2 sec  | 1995^1.1 
Retry #4 delayed by 9.8 sec  | 4265^1.1 
Retry #5 delayed by 24 sec   | 9865^1.1
*/
