'use strict';
const {
  Model
} = require('sequelize');
module.exports = (sequelize, DataTypes) => {
  class File extends Model {
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models) {
      // define association here
      File.hasMany(models.FileChunk,{
        onDelete: 'RESTRICT'
      });
      File.hasOne(models.Job,{
        onDelete: 'RESTRICT'
      });
      File.belongsTo(models.User, {
        foreignKey: {
          name : "createdBy",
          allowNull: false
        },
        as:"fileCreator"
      });
    }
  };

  File.init({
    id: {
        type: DataTypes.UUID,
        defaultValue: DataTypes.UUIDV4,
        primaryKey: true,
        allowNull: false,
    },
    name: {
      type: DataTypes.STRING,
      allowNull: false
    },
    type: {
      type: DataTypes.STRING,
      validate: {
        isIn: {
            args: [
              ['Import', 'Export']
            ],
            msg: "Type must be 'Import' or 'Export'"
        }
      },
      allowNull: false,
    },
    format: {
      type: DataTypes.STRING,
      allowNull: false
    },
    location: {
      type: DataTypes.STRING,
      allowNull: false
    },
    mapping: {
      type: DataTypes.JSON,
      allowNull: false
    }
  }, {
    sequelize,
    modelName: 'File',
    timestamps: true,
  });
  return File;
};