"use strict";
const { Model } = require("sequelize");
module.exports = (sequelize, DataTypes) => {
  class Job extends Model {
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models) {
      // define association here 
      Job.hasMany(models.JobError,{
        onDelete: 'RESTRICT'
      });
      Job.belongsTo(models.File, {
        onDelete: "RESTRICT",
        foreignKey: {
          unique: true,
          allowNull: false
        },
      });
      Job.belongsTo(models.User, {
        foreignKey: {
          name : "createdBy",
          allowNull: false
        },
        as:"jobCreator"
      });
    }
  }
  Job.init(
    {
      id: {
        type: DataTypes.UUID,
        defaultValue: DataTypes.UUIDV4,
        primaryKey: true,
        allowNull: false,
      },
      status: {
        type: DataTypes.STRING,
        validate: {
          isIn: {
              args: [
                ["Queued", "Processing", "Completed", "Failed", "Preparing", "Verifying", "Cancelled", "Verified"]
              ],
              msg: "Status must be 'Queued', 'Processing', 'Preparing', 'Completed', 'Verifying', 'Verified', 'Cancelled' or 'Failed'"
          }
        },
        allowNull: false,
      },
      chunks: {
        type: DataTypes.INTEGER,
        defaultValue: 0
      },
      rowCount: {
        type: DataTypes.INTEGER,
        defaultValue: 0
      },
      operationName: {
        type: DataTypes.STRING,
        validate: {
          isIn: {
              args: [
                [
                  "accountImport",
                  "accountMetaImport",
                  "accountTechnologyImport",
                  "contactImport",
                  "syncAccountExport", 
                  "asyncAccountExport", 
                  "syncContactExport",
                  "asyncContactExport",
                  "asyncVerifyAccountExport"
                ]
              ],
              msg: "Operation Name must be 'accountImport', 'accountMetaImport', 'accountTechnologyImport', 'contactImport','syncAccountExport','asyncAccountExport','syncContactExport' or 'asyncContactExport'"
          }
        },
        allowNull: false,
      },
      operationParam: {
        type: DataTypes.JSON
      },
      processed: {
        type: DataTypes.INTEGER,
        defaultValue:0
      },
      imported: {
        type: DataTypes.INTEGER,
        defaultValue:0
      },
      errored: {
        type: DataTypes.INTEGER,
        defaultValue:0
      },
    },
    {
      sequelize,
      modelName: "Job",
      timestamps: true,
      indexes: [
      ],
    }
  );
  return Job;
};
