"use strict";
const {
  Model
} = require("sequelize");
module.exports = (sequelize, DataTypes) => {
  class JobError extends Model {
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models) {
      // define association here
      JobError.belongsTo(models.Job, {
        onDelete: "RESTRICT",
        foreignKey: {
          allowNull: false,
        },
      });
    }
  }
  JobError.init({
    id: {
        type: DataTypes.UUID,
        defaultValue: DataTypes.UUIDV4,
        primaryKey: true,
        allowNull: false,
    },
    description: {
      type: DataTypes.TEXT,
    },
    rowContent: {
      type: DataTypes.JSON,
      allowNull: false
    },
    rowCount: {
      type: DataTypes.INTEGER,
      defaultValue: 0
    },
    type: {
      type: DataTypes.STRING,
      validate: {
        isIn: {
            args: [
              ["row", "chunk", "job"]
            ],
            msg: "Type must be 'row', 'job' or 'chunk'"
        }
      },
      allowNull: false
    },
    rowIndex: {
      type: DataTypes.INTEGER,
      defaultValue: 0
    },
    chunkIndex: {
      type: DataTypes.INTEGER,
      defaultValue: 0
    },
  }, {
    sequelize,
    modelName: "JobError",
    timestamps: true
  });
  return JobError;
};