"use strict";
const { Model } = require("sequelize");
const { LOCATION_TYPES } = require("../../services/constants");
const locationTypes = Object.values(LOCATION_TYPES)
module.exports = (sequelize, DataTypes) => {
  class Location extends Model {
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models) {
      // define association here
      Location.belongsTo(models.Account, {
        onDelete: "RESTRICT",
        foreignKey: {
          allowNull: false,
        },
      }),
        Location.belongsTo(models.User, {
          foreignKey: {
            name: "createdBy",
            allowNull: false,
          },
          as: "locationCreator",
        });
      Location.belongsTo(models.User, {
        foreignKey: {
          name: "updatedBy",
          allowNull: false,
        },
        as: "locationUpdater",
      });
    }
  }
  Location.init(
    {
      id: {
        type: DataTypes.UUID,
        defaultValue: DataTypes.UUIDV4,
        primaryKey: true,
        allowNull: false,
      },
      type: {
        type: DataTypes.STRING,
        validate: {
          isIn: {
            args: [locationTypes],
            msg: `Type must be ${locationTypes.join(",")}`,
          },
        },
        allowNull: false,
      },
      address1: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      address2: {
        type: DataTypes.STRING,
      },
      city: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      state: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      // auto generate based on the state
      stateAbbreviation: {
        type: DataTypes.STRING,
      },
      zipCode: {
        type: DataTypes.STRING,
      },
      country: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      // auto generate based on the timeZone
      timeZone: {
        type: DataTypes.STRING,
      },
      phone1: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      phone2: {
        type: DataTypes.STRING,
      },
      // address1(remove special character)+city+state+country -> (remove space, lowercase)
      addressDedupeKey: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      accountDomain: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      // auto generate based on the country
      region: {
        type: DataTypes.STRING,
      },
    },
    {
      sequelize,
      modelName: "Location",
      timestamps: true,
      indexes: [
        {
          fields: ["state"],
        },
        {
          fields: ["zipCode"],
        },
        {
          fields: ["country"],
        },
        {
          fields: ["timeZone"],
        },
        {
          fields: ["addressDedupeKey"],
        },
        {
          fields: ["accountDomain"],
        },
        {
          fields: ["type", "accountDomain"],
        }
      ],
    }
  );
  return Location;
};
