"use strict";
const { Model } = require("sequelize");

module.exports = (sequelize, DataTypes) => {
  
  class User extends Model {
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models) {
      // define association here
      User.hasMany(models.Account,{
        foreignKey: "createdBy",
        onDelete: "RESTRICT",
        as: "accountCreator",
      });
      User.hasMany(models.Account,{
        foreignKey: "updatedBy",
        onDelete: "RESTRICT",
        as: "accountUpdater",
      });
      User.hasMany(models.Location,{
        foreignKey: "createdBy",
        onDelete: "RESTRICT",
        as: "locationCreator",
      });
      User.hasMany(models.Location,{
        foreignKey: "updatedBy",
        onDelete: "RESTRICT",
        as: "locationUpdater",
      });
      User.hasMany(models.File,{
        foreignKey: "createdBy",
        onDelete: "RESTRICT",
        as: "fileCreator",
      });
      User.hasMany(models.Job,{
        foreignKey: "createdBy",
        onDelete: "RESTRICT",
        as: "jobCreator",
      });
    }
  }
  User.init(
    {
      //unique & notNull
      id: {
        type: DataTypes.STRING,
        unique: true,
        allowNull: false,
      },
      firstName: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      lastName: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      role: {
        type: DataTypes.ARRAY(DataTypes.STRING),
        validate: {
          isIn: {
            args: [
              ["manager", "compliance"]
            ],
            msg: "role must be 'manager' or 'compliance'"
          }
        },
        allowNull: false,
      },
      emailId: {
        type: DataTypes.STRING,
        primaryKey: true,
        allowNull: false,
        // checks for email format
        validate: {
          isEmail: true,
        }
      }
    },
    {
      sequelize,
      modelName: "User",
      timestamps: true,
      indexes: [
        {
          fields: ["role"],
          using: "gin",
        }
      ]
    }
  );
  return User;
};
