/* eslint-disable no-console */
const { Pool } = require('pg');
const QueryStream = require('pg-query-stream');
const { serializeError } = require('serialize-error');

const dbSocketPath = process.env.DB_SOCKET_PATH || '/cloudsql';
const config = {
  host: `${dbSocketPath}/${process.env.MASTER_DB_INSTANCE_URL}`,
  user: process.env.DB_USER,
  port: 5432,
  database: process.env.DB_NAME,
  password: "HDMI6ha0wTBj11vEmMc24U",
  max: 10,
  min: 0,
  idleTimeoutMillis: 10,
  log: (messages) => {
    console.log(messages);
  },
  allowExitOnIdle: true,
};

const pool = new Pool(config);

async function connectDb() {
  try {
    const connection = await pool.connect();
    return connection;
  } catch (error) {
    error.code = 'MASTER_DB_CONNECTION_ERROR';
    const connection = {};
    connection.error = error;
    return connection;
  }
}

function getStreamyQuery(sql, param) {
  return new QueryStream(sql, param);
}

module.exports = {
  connectDb,
  queryStreamer: (sql, dataConsumerStream, sqlArgs = []) =>
    new Promise((resolve, reject) => {
      console.log('[MASTER_QUERY_STREAMER] :: [DB_CONNECTION] : Connecting Client');

      connectDb().then((connection) => {
        if (connection.error) {
          console.log('[MASTER_QUERY_STREAMER] :: [DB_CONNECTION] : Client Connection Error');
          console.log(JSON.stringify(serializeError(connection.error)));
          reject(connection.error);
        }
        console.log('[MASTER_QUERY_STREAMER] :: [DB_CONNECTION] : Client Connected Successfully');

        const streamyQuery = getStreamyQuery(sql, sqlArgs);

        const dataProducerStream = connection.query(streamyQuery);

        dataProducerStream.on('error', (error) => {
          console.log('[MASTER_QUERY_STREAMER] :: [DATA_PRODUCER_STREAM] : Stream Error');
          console.log(JSON.stringify(serializeError(error)));
        });

        dataProducerStream.on('end', () => {
          console.log('[MASTER_QUERY_STREAMER] :: [DATA_PRODUCER_STREAM] : Stream Ended');
          connection.release();
          resolve();
        });

        dataProducerStream.pipe(dataConsumerStream);
      });
    }),
};