#!/bin/bash
set -e

# https://sequelize.org/master/manual/migrations.html

# npx sequelize-cli init
# npx sequelize-cli model:generate --name User --attributes firstName:string,lastName:string,email:string
# npx sequelize-cli db:migrate
# npx sequelize-cli db:migrate:undo
# npx sequelize-cli db:migrate:undo:all --to XXXXXXXXXXXXXX-create-posts.js
# npx sequelize-cli seed:generate --name demo-user
# npx sequelize-cli db:seed:all
# npx sequelize-cli db:seed:undo
# npx sequelize-cli db:seed:undo --seed name-of-seed-as-in-data
# npx sequelize-cli db:seed:undo:all
# npx sequelize-cli migration:generate --name migration-skeleton

#  The default Postgres username is "postgres"

POSTGRES_PASSWORD='HDMI6ha0wTBj11vEmMc24U'
MASTER_DB_INSTANCE_URL='localhost'
DB_USER='postgres'

# Creates directory
if [ ! -d ${PWD}/postgres-data/ ]
then
    mkdir ${PWD}/postgres-data/

else
    if [ "$(docker ps -q -f name=dev-pgadmin)" ]
    then
        docker stop dev-pgadmin # stops dev-pgadmin docker container
    fi

    if [ "$(docker ps -q -f name=dev-postgres)" ]
    then
        docker stop dev-postgres # stops dev-postgres docker container
    fi

    if [ "$(docker ps -aq -f status=exited -f name=dev-postgres)" ]; then
        # cleanup
        docker rm -v dev-postgres # removes dev-postgres docker container and volumes associated
    fi

    if [ "$(docker ps -aq -f status=exited -f name=dev-pgadmin)" ]; then
        # cleanup
        docker rm -v dev-pgadmin # removes dev-pgadmin docker container and volumes associated
    fi
    echo "Please, delete Directory :: ${PWD}/postgres-data/"
    exit 1 # If directory ${PWD}/postgres-data/ exists, return exit code 1
fi

# Check for .nvm and install node v14.16.1 using nvm
if [ ! -d ~/.nvm ]
then
    mkdir ${HOME}/.nvm
    curl -o- https://raw.githubusercontent.com/creationix/nvm/v0.39.1/install.sh | bash
    export NVM_DIR="$HOME/.nvm"
    [ -s "$NVM_DIR/nvm.sh" ] && \. "$NVM_DIR/nvm.sh"
    [ -s "$NVM_DIR/bash_completion" ] && \. "$NVM_DIR/bash_completion"
fi
source ~/.nvm/nvm.sh

nvm install 16.13.1
nvm use 16.13.1
npm install # Install packages from package.json

# Pull docker images
docker pull postgres
docker pull dpage/pgadmin4

if [ ! "$(docker ps -q -f name=dev-postgres)" ]; then
    if [ "$(docker ps -aq -f status=exited -f name=dev-postgres)" ]; then
        # cleanup
        docker rm -v dev-postgres
    fi
    # run your container
    docker run -d --name dev-postgres -e POSTGRES_PASSWORD=$POSTGRES_PASSWORD -v ${PWD}/postgres-data/:/var/lib/postgresql/data -p 5432:5432 postgres
fi

if [ ! "$(docker ps -q -f name=dev-pgadmin)" ]; then
    if [ "$(docker ps -aq -f status=exited -f name=dev-pgadmin)" ]; then
        # cleanup
        docker rm -v dev-pgadmin
    fi
    # run your container
    docker run -p 8090:80 -e 'PGADMIN_DEFAULT_EMAIL=admin@admin.com' -e 'PGADMIN_DEFAULT_PASSWORD=root' --name dev-pgadmin -d dpage/pgadmin4
fi

sleep 10 # waits for 10 sec

# Print IP of Postgres (used in PGAdmin GUI)
ip=$(docker inspect dev-postgres -f "{{json .NetworkSettings.Networks.bridge.IPAddress }}")
echo "$ip"

# Selects DB name based on NODE_ENV
if [ "$NODE_ENV" == 'production' ]
then
    db_name='master_db_prod'
    SEEDERS_PATH='lib/db/productionSeeders'
else
    db_name='master_db_dev'
    SEEDERS_PATH='lib/db/developmentSeeders'
fi

# Check for existence of DB and create DB
db_exists=$(docker exec -it dev-postgres psql -U postgres -l | grep $db_name | wc -l)

if [ $db_exists == 0 ]
then
    docker exec -it dev-postgres psql -U postgres -c "CREATE DATABASE $db_name;"
else
    docker exec -it dev-postgres psql -U postgres -c "DROP DATABASE $db_name;"
    docker exec -it dev-postgres psql -U postgres -c "CREATE DATABASE $db_name;"
fi

# Sync or Drop Models based on $OPERATION
if [ "$OPERATION" == 'create' ]
then
    NODE_ENV=local DB_USER=$DB_USER DB_NAME=$db_name MASTER_DB_INSTANCE_URL=$MASTER_DB_INSTANCE_URL DATABASE_PASSWORD=$POSTGRES_PASSWORD node ./syncModels.js

    # Run migrations for creating tables and their associations
    # NODE_ENV=local DB_USER=$DB_USER DB_NAME=$db_name MASTER_DB_INSTANCE_URL=$MASTER_DB_INSTANCE_URL DATABASE_PASSWORD=$POSTGRES_PASSWORD npx sequelize-cli db:migrate

    # Seeds all tables with some data
    NODE_ENV=local DB_USER=$DB_USER DB_NAME=$db_name MASTER_DB_INSTANCE_URL=$MASTER_DB_INSTANCE_URL DATABASE_PASSWORD=$POSTGRES_PASSWORD npx sequelize-cli db:seed:all --seeders-path $SEEDERS_PATH

else
    NODE_ENV=local DB_USER=$DB_USER DB_NAME=$db_name MASTER_DB_INSTANCE_URL=$MASTER_DB_INSTANCE_URL DATABASE_PASSWORD=$POSTGRES_PASSWORD node ./dropModels.js
fi
